//	BeStrip.cc

#include "BeStrip.h"

const rgb_color StripWindow::baseColor = { 0xD8, 0xD8, 0xD8, 0xFF };

const char *StripApplication::backupText = "BeStrip, version 1.0, by Ernest Tomlinson";
const char *StripApplication::moduleFolderName = "modules";

ModuleFilter::ModuleFilter(BeModule *handler)
	: BMessageFilter(B_ANY_DELIVERY,B_ANY_SOURCE)
{
	module = handler;
}

filter_result ModuleFilter::Filter(BMessage *message, BHandler **target)
{
	ulong flags = module->ModuleFlags();
	if (message->what == B_MOUSE_DOWN && ((flags & BM_HIGHLIGHT_BUTTON) || !(flags & BM_HANDLE_MOUSE)))
	{
		BMessage *newMsg = new BMessage(message);
		newMsg->AddBool("passthru",true);
		module->Looper()->PostMessage(newMsg,module->Parent());
		if (!(flags & BM_HANDLE_MOUSE))
			return B_SKIP_MESSAGE;
	}
	return B_DISPATCH_MESSAGE;
}

ModuleContainer::ModuleContainer(BeModule *child, rgb_color baseColor, image_id image)
	: PushButton(BRect(),NULL,NULL,NULL,NULL,false,B_FOLLOW_NONE,B_WILL_DRAW | B_FRAME_EVENTS)
{
	addon = image;
	module = child;
	AddChild(module);
	colors = new ShadowColors(baseColor);
	SetViewColor(colors->normal);
	module->SetViewColor(colors->normal);
	if (module->ModuleFlags() & BM_NOTIFY_ON_RELEASE)
		SetInvokeOnRelease(true);
}

ModuleContainer::~ModuleContainer()
{
	RemoveChild(module);
	unload_add_on(addon);
}

void ModuleContainer::calcPics()
{
	rgb_color color;
	BPicture *picture = new BPicture;
	BeginPicture(picture);
	BRect box = Bounds();
	SetHighColor(colors->darker);
	StrokeLine(box.LeftBottom(),box.RightBottom());
	StrokeLine(box.RightTop(),box.RightBottom());
	SetHighColor(colors->light);
	StrokeLine(box.LeftTop(),box.LeftBottom());
	StrokeLine(box.LeftTop(),box.RightTop());
	box.InsetBy(1,1);
	SetHighColor(colors->dark);
	StrokeLine(box.LeftBottom(),box.RightBottom());
	StrokeLine(box.RightTop(),box.RightBottom());
	color.red = color.blue = color.green = color.alpha = 0xFF;
	SetHighColor(color);
	StrokeLine(box.LeftTop(),box.LeftBottom());
	StrokeLine(box.LeftTop(),box.RightTop());
	EndPicture();
	SetOffPicture(picture);
	picture = new BPicture;
	BeginPicture(picture);
	box = Bounds();
	SetHighColor(colors->normal);
	StrokeLine(box.LeftBottom(),box.RightBottom());
	StrokeLine(box.RightTop(),box.RightBottom());
	SetHighColor(colors->darker);
	StrokeLine(box.LeftTop(),box.LeftBottom());
	StrokeLine(box.LeftTop(),box.RightTop());
	box.InsetBy(1,1);
	SetHighColor(colors->normal);
	StrokeLine(box.LeftBottom(),box.RightBottom());
	StrokeLine(box.RightTop(),box.RightBottom());
	SetHighColor(colors->darker);
	StrokeLine(box.LeftTop(),box.LeftBottom());
	StrokeLine(box.LeftTop(),box.RightTop());
	box.InsetBy(1,1);
	SetHighColor(colors->dark);
	FillRect(box);
	EndPicture();
	SetOnPicture(picture);
}

void ModuleContainer::AttachedToWindow()
{
	if (!(module->ModuleFlags() & BM_HANDLE_MOUSE))
	{
		SetMessage(new BMessage(BM_CLICKED));
		SetTarget(module);
	}
	Window()->Lock();
	module->AddFilter(new ModuleFilter(module));
	Window()->Unlock();
	calcPics();
}

void ModuleContainer::FrameMoved(BPoint where)
{
	BRect box = module->Bounds();
	float left = (Bounds().Width()-box.Width())/2;
	float top = (Bounds().Height()-box.Height())/2;
	module->MoveTo(left,top);
}
	
void ModuleContainer::FrameResized(float width, float height)
{
	calcPics();
	BRect box = module->Bounds();
	float left = (Bounds().Width()-box.Width())/2;
	float top = (Bounds().Height()-box.Height())/2;
	module->MoveTo(left,top);
}

void ModuleContainer::MessageReceived(BMessage *message)
{
	if (message->what == B_MOUSE_UP && !(module->ModuleFlags() & BM_HIGHLIGHT_BUTTON))
		Invoke();
	PushButton::MessageReceived(message);
}

void ModuleContainer::MouseDown(BPoint where)
{
	ulong flags = module->ModuleFlags();
	if (flags & BM_HIGHLIGHT_BUTTON)
	{
		if (flags & BM_HANDLE_MOUSE)
		{
			BMessage *message = Looper()->CurrentMessage();
			message->FindBool("passthru");
			if (message->Error() != B_NO_ERROR)
				return;
		}
		PushButton::MouseDown(where);
	}
	else
		Invoke();
}

StripWindow::StripWindow(BPoint where)
	: Infinitas(BRect(where,where),NULL,baseColor,B_NOT_RESIZABLE | B_NOT_CLOSABLE | B_WILL_ACCEPT_FIRST_CLICK,false)
{
	moduleList = new BList;
	recalc();
}

StripWindow::~StripWindow()
{
	delete moduleList;
}

void StripWindow::recalc()
{
	BRect bounds;
	ModuleContainer *tile;
	Lock();
	float width = (IsHorizontal() ? STRIP_MIN_WIDTH : 0);
	float height = (IsHorizontal() ? 0 : STRIP_MIN_WIDTH);
	for (long i = 0; i < moduleList->CountItems(); i++)
	{
		tile = (ModuleContainer *)moduleList->ItemAt(i);
		bounds = tile->module->Bounds();
		bounds.InsetBy(-STRIP_MODULE_BORDER,-STRIP_MODULE_BORDER);
		width = (bounds.Width() > width ? bounds.Width() : width);
		height = (bounds.Height() > height ? bounds.Height() : height);
	}
	BRect prevBounds;
	if (IsHorizontal())
		prevBounds.Set(0,0,width,I_TITLEBAR_THICKNESS);
	else
		prevBounds.Set(0,0,I_TITLEBAR_THICKNESS,height);
	bounds = prevBounds;
	for (long i = 0; i < moduleList->CountItems(); i++)
	{
		tile = (ModuleContainer *)moduleList->ItemAt(i);
		bounds = tile->module->Bounds();
		bounds.InsetBy(-STRIP_MODULE_BORDER,-STRIP_MODULE_BORDER);
		if (IsHorizontal())
		{
			bounds.OffsetTo(prevBounds.left,prevBounds.bottom+1);
			bounds.right = bounds.left+width;
		}
		else
		{
			bounds.OffsetTo(prevBounds.right+1,prevBounds.top);
			bounds.bottom = bounds.top+height;
		}
		tile->MoveTo(bounds.LeftTop());
		tile->ResizeTo(bounds.Width(),bounds.Height());
		prevBounds = bounds;
	}
	ResizeTo(bounds.right,bounds.bottom);
	Unlock();
}

void StripWindow::DispatchMessage(BMessage *message, BHandler *handler)
{
	switch (message->what)
	{
		case BM_CLICKED:
		case B_MOUSE_UP:
			BLooper::DispatchMessage(message,handler);
			break;
		case B_VIEW_RESIZED:
			if (is_kind_of(handler,BeModule))
				recalc();
		default:
			Infinitas::DispatchMessage(message,handler);
			break;
	}
}

void StripWindow::Minimize(bool doMinimize)
{
	WindowShade(doMinimize);
}

void StripWindow::Zoom(BPoint where, float width, float height)
{
	Flip();
	recalc();
}

long StripWindow::AddModule(BFile *file)
{
	BeModule *(*createProc)(BResourceFile *);
	image_id image = load_add_on(file);
	if (image == B_ERROR)
		return image;
	long err = get_image_symbol(image,"CreateModule",B_SYMBOL_TYPE_TEXT,&createProc);
	if (err != B_NO_ERROR)
		return err;
	BResourceFile *resFile = new BResourceFile(file->Record()->Ref());
	BeModule *module = createProc(resFile);
	if (!module)
	{
		delete resFile;
		return B_ERROR;
	}
	ModuleContainer *tile = new ModuleContainer(module,baseColor,image);
	moduleList->AddItem(tile);
	AddChild(tile);
	recalc();
	return B_NO_ERROR;
}

StripApplication::StripApplication()
	: BApplication(StripApplication::signature)
{
}

void StripApplication::AboutRequested()
{
	char *text;
	if (aboutText)
		text = aboutText;
	else
		text = backupText;
	BAlert *alert = new BAlert("BeStrip",text,"OK");
	alert->Go();
}

void StripApplication::ReadyToRun()
{
	BFile appFile;
	BFile moduleFile;
	BResourceFile appResFile;
	BDirectory appFolder;
	BDirectory moduleFolder;
	record_ref moduleFolderRef;
	record_ref *moduleRef = NULL;
	long numModules, length;
	screen_info screenInfo;
	get_screen_info(&screenInfo);
	BRect frame = screenInfo.frame;
	float dockWidth;
	long err = get_dock_width(&dockWidth);
	if (err == B_NO_ERROR)
		frame.left += dockWidth;
	StripWindow *window = new StripWindow(BPoint(frame.left+5,frame.top+5));
	app_info appInfo;
	err = GetAppInfo(&appInfo);
	if (err != B_NO_ERROR)
		goto bail;
	appFile.SetRef(appInfo.ref);
	appResFile.SetRef(appInfo.ref);
	err = appResFile.Open(B_READ_ONLY);
	if (err == B_NO_ERROR)
	{
		aboutText = (char *)appResFile.FindResource((ulong)aboutTextResType,(long)aboutTextResID,&length);
		appResFile.Close();
	}
	err = appFile.GetParent(&appFolder);
	if (err != B_NO_ERROR)
		goto bail;
	err = appFolder.GetRefForPath(StripApplication::moduleFolderName,&moduleFolderRef);
	if (err != B_NO_ERROR)
	{
		err = appFolder.Create(StripApplication::moduleFolderName,&moduleFolder);
		if (err != B_NO_ERROR)
			goto bail;
	}
	else
		moduleFolder.SetRef(moduleFolderRef);
	numModules = moduleFolder.CountFiles();
	moduleRef = new record_ref[numModules];
	err = moduleFolder.GetFiles(0,numModules,moduleRef);
	if (err != B_NO_ERROR)
		goto bail;
	for (long i = 0; i < numModules; i++)
	{
		moduleFile.SetRef(moduleRef[i]);
		window->AddModule(&moduleFile);
	}
bail:
	if (moduleRef)
		delete moduleRef;
	window->Show();
}