//	ColorPopup.cc

#include "ColorPopup.h"

ColorItem::ColorItem(float size, rgb_color color)
	: BMenuItem("",NULL,NULL,NULL)
{
	rgbColor = color;
	boxWidth = size;
	BMessage *message = new BMessage(CP_GOT_COLOR);
	message->AddData("color",B_RGB_COLOR_TYPE,&rgbColor,sizeof(rgbColor));
	SetMessage(message);
}

void ColorItem::DrawContent(void)
{
	float width, height;
	rgb_color high = Menu()->HighColor();
	BPoint corner = ContentLocation();
	GetContentSize(&width,&height);
	BRect bounds(corner.x,corner.y,corner.x+width,corner.y+height);
	bounds.InsetBy(1,1);
	Menu()->SetHighColor(rgbColor);
	Menu()->FillRect(bounds);
	Menu()->SetHighColor(high);
}

void ColorItem::GetContentSize(float *width, float *height)
{
	*width = boxWidth;
	*height = boxWidth;
}

void ColorItem::Highlight(bool isHilit)
{
	drawing_mode drawMode = Menu()->DrawingMode();
	Menu()->SetDrawingMode(B_OP_INVERT);
	Menu()->StrokeRect(Frame());
	Menu()->SetDrawingMode(drawMode);
}

rgb_color ColorItem::Color()
{
	return rgbColor;
}

ColorPopup::ColorPopup(const char *name, float boxWidth)
	: MatrixPopup(name,16*boxWidth,16*boxWidth)
{
	BRect bounds(0,0,boxWidth,boxWidth);
	color_map *clut = system_colors();
	for (int row = 0; row < 16; row++)
		for (int column = 0; column < 16; column++)
		{
			bounds.OffsetTo(column*boxWidth,row*boxWidth);
			AddItem(new ColorItem(boxWidth,clut->color_list[row*16+column]),bounds);
		}	
}

BPoint ColorPopup::ScreenLocation()
{
	return MatrixPopup::ScreenLocation();
}