//	BMUtils.cc

#include "BMUtils.h"

BBitmap *bm_get_bitmap(uchar *,BRect);

BBitmap *bm_get_bitmap(uchar *data, BRect bounds)
{
	BBitmap *bitmap = new BBitmap(bounds,B_COLOR_8_BIT);
	for (long rows = 0, offset = 0, index = 0; rows <= bounds.Height(); rows++, offset += bitmap->BytesPerRow(), index += (bounds.Width()+1))
		bitmap->SetBits(data+index,1+bounds.Width(),offset,B_COLOR_8_BIT);
	return bitmap;
}

BBitmap *BMLoadBitmapData(uchar *data, long length)
{
	long offset = 0;
	ulong left, right, top, bottom, numPixels;
	memcpy(&left,data+offset,4);
	offset += 4;
	if (offset >= length)
		return NULL;
	memcpy(&top,data+offset,4);
	offset += 4;
	if (offset >= length)
		return NULL;
	memcpy(&right,data+offset,4);
	offset += 4;
	if (offset >= length)
		return NULL;
	memcpy(&bottom,data+offset,4);
	offset += 4;
	if (offset >= length)
		return NULL;
//	bit depth field is ignored for the moment
	offset += 4;
	if (offset >= length)
		return NULL;
	memcpy(&numPixels,data+offset,4);
	offset += 4;
	if (offset+numPixels != length)
		return NULL;
	else
		return bm_get_bitmap(data+offset,BRect(left,top,right,bottom));
}

BBitmap *BMLoadBitmapFile(BFile *file)
{
	long length = file->Size();
	if (length == B_ERROR)
		return NULL;
	long err = file->Open(B_READ_ONLY);
	if (err != B_NO_ERROR)
		return NULL;
	uchar *data = new uchar[length];
	long n = file->Read(data,length);
	if (n == B_ERROR || n != length)
	{
		file->Close();
		return NULL;
	}
	BBitmap *bitmap = BMLoadBitmapData(data,length);
	file->Close();
	return bitmap;
}

BBitmap *BMLoadBitmapRsrc(BResourceFile *resFile, long resID)
{
	long err = resFile->Open(B_READ_ONLY);
	if (err != B_NO_ERROR)
		return NULL;
	long length;
	uchar *data = (uchar *)resFile->FindResource('CLRB',resID,&length);
	if (!data)
	{
		resFile->Close();
		return NULL;
	}
//	note:  the CLRB resource as produced by the VM_Plugins editor
//	has the first 2 fields (each 4 bytes) screwed up in some odd
//	way.  This is a makeshift attempt to correct for that.
	memset(data,0,8);
	BBitmap *bitmap = BMLoadBitmapData(data,length);
	resFile->Close();
	return bitmap;
}

BBitmap *BMLoadLargeIcon(BResourceFile *resFile, long resID)
{
	long err = resFile->Open(B_READ_ONLY);
	if (err != B_NO_ERROR)
		return NULL;
	long length;
	uchar *data = (uchar *)resFile->FindResource('ICON',resID,&length);
	if (!data)
	{
		resFile->Close();
		return NULL;
	}
	BBitmap *bitmap = bm_get_bitmap(data,BRect(0,0,31,31));
	resFile->Close();
	return bitmap;
}

BBitmap *BMLoadSmallIcon(BResourceFile *resFile, long resID)
{
	long err = resFile->Open(B_READ_ONLY);
	if (err != B_NO_ERROR)
		return NULL;
	long length;
	uchar *data = (uchar *)resFile->FindResource('MICN',resID,&length);
	if (!data)
	{
		resFile->Close();
		return NULL;
	}
	BBitmap *bitmap = bm_get_bitmap(data,BRect(0,0,15,15));
	resFile->Close();
	return bitmap;
}

void BMDrawIcon(BView *view, BBitmap *icon, BPoint where, bool selected)
{
	drawing_mode drawMode = view->DrawingMode();
	if (selected)
	{
		view->SetDrawingMode(B_OP_INVERT);
		view->DrawBitmap(icon,where);
		view->SetDrawingMode(B_OP_BLEND);
	}
	else
		view->SetDrawingMode(B_OP_OVER);
	view->DrawBitmap(icon,where);
	view->SetDrawingMode(drawMode);
}

bool BMBitmapHit(BBitmap *icon, BPoint where)
{
	if (icon->Bounds().Contains(where))
	{
		uchar *bits = (uchar *)icon->Bits();
		int x = floor(where.x);
		int y = floor(where.y);
		if (bits[y*icon->BytesPerRow()+x] != B_TRANSPARENT_8_BIT)
			return true;
	}
	return false;
}

MatrixPopup::MatrixPopup(const char *name, float width, float height)
	: BMenu(name,width,height)
{
}

BPoint MatrixPopup::ScreenLocation()
{
	return place;
}

BMenuItem *MatrixPopup::Go(BPoint where, bool sendMsg, bool openAnyway)
{
	place = where;
	Show();
	BMenuItem *result = Track(openAnyway);
	Hide();
	if (sendMsg && result && result->Target() && result->Target()->Looper())
		result->Target()->Looper()->PostMessage(result->Message(),result->Target());
	return result;
}

BMenuItem *MatrixPopup::Go(BRect box, bool sendMsg, bool openAnyway)
{
	screen_info screenInfo;
	get_screen_info(&screenInfo);
	BRect bounds = screenInfo.frame;
	BRect menuBounds = Frame();
	BPoint corner;
	BPoint boxCenter((box.right+box.left)/2,(box.top+box.bottom)/2);
	BPoint screenCenter((bounds.left+bounds.right)/2,(bounds.top+bounds.bottom)/2);
	if (boxCenter.x < screenCenter.x)
	{
		if (boxCenter.y < screenCenter.y)
			corner = box.LeftBottom();
		else
		{
			corner = box.LeftTop();
			corner.y -= menuBounds.Height();
		}
	}
	else
	{
		if (boxCenter.y < screenCenter.y)
		{
			corner = box.RightBottom();
			corner.x -= menuBounds.Width();
		}
		else
		{
			corner = box.RightTop();
			corner.x -= menuBounds.Width();
			corner.y -= menuBounds.Height();
		}
	}
	return Go(corner,sendMsg,openAnyway);
}

BMenuItem *MatrixPopup::Go(BPoint where, bool sendMsg, bool openAnyway, BRect specialRect)
{
	BRect rect = specialRect;
	place = where;
	Show();
	BMenuItem *result = Track(openAnyway,&rect);
	Hide();
	return result;
}
