//	PushButton.cc

#include "PushButton.h"

PushButton::PushButton(BRect bounds, const char *name, BPicture *on, BPicture *off, BMessage *message, bool onRelease, ulong resizeMode, ulong flags)
	: BControl(bounds,name,"",message,resizeMode,flags)
{
	onPic = on;
	offPic = off;
	invokeOnRelease = onRelease;
}

PushButton::~PushButton()
{
	if (onPic)
		delete onPic;
	if (offPic)
		delete offPic;
}

void PushButton::Draw(BRect clipRect)
{
	if (Value() == B_CONTROL_OFF)
	{
		if (offPic)
			DrawPicture(offPic,BPoint(0,0));
	}
	else
	{
		if (onPic)
			DrawPicture(onPic,BPoint(0,0));
	}
}

long PushButton::threadProc(void *object)
{
	BPoint where;
	ulong buttons;
	PushButton *button = (PushButton *)object;
	BWindow *window = button->Window();
	if (!button->invokeOnRelease)
	{
		window->Lock();
		button->SetValue(B_CONTROL_ON);
		button->Invalidate();
		window->Unlock();
		button->Invoke();
	}
	do
	{
		if (button->invokeOnRelease)
		{
			window->Lock();
			button->GetMouse(&where,&buttons);
			if (button->Bounds().Contains(where) && button->Value() == B_CONTROL_OFF)
			{
				button->SetValue(B_CONTROL_ON);
				button->Invalidate();
			}
			else if (!button->Bounds().Contains(where) && button->Value() != B_CONTROL_OFF)
			{
				button->SetValue(B_CONTROL_OFF);
				button->Invalidate();
			}
			window->Unlock();
		}
		snooze(30000);
	}
	while (buttons);
	long value = button->Value();
	window->Lock();
	button->SetValue(B_CONTROL_OFF);
	button->Invalidate();
	window->Unlock();
	if (value != B_CONTROL_OFF && button->invokeOnRelease)
		button->Invoke();
	return 0;
}

void PushButton::MouseDown(BPoint where)
{
	thread_id thread = spawn_thread(&threadProc,"push_button_thread",B_NORMAL_PRIORITY,this);
	resume_thread(thread);
}

void PushButton::SetInvokeOnRelease(bool onRelease)
{
	invokeOnRelease = onRelease;
}

void PushButton::SetOnPicture(BPicture *on)
{
	if (Window())
		Window()->Lock();
	if (onPic)
		delete onPic;
	onPic = on;
	if (Window())
		Window()->Unlock();
	if (Value() != B_CONTROL_OFF)
		Invalidate();
}

void PushButton::SetOffPicture(BPicture *off)
{
	if (Window())
		Window()->Lock();
	if (offPic)
		delete offPic;
	offPic = off;
	if (Window())
		Window()->Unlock();
	if (Value() == B_CONTROL_OFF)
		Invalidate();
}